@@ -84,6 +84,7 @@
 //!   - Full of `unsafe`
 //!
 //! # Changelog
+//! - v0.2.2 - Add `ThreadPool::join` and `ThreadPool::try_join`.
 //! - v0.2.1 - Improve test coverage.
 //! - v0.2.0
 //!   - `ThreadPool::new` to return `Result`.
@@ -114,10 +115,15 @@ use core::time::Duration;
 use std::error::Error;
 use std::sync::mpsc::{Receiver, RecvTimeoutError, SyncSender, TrySendError};
 use std::sync::{Arc, Mutex};
+use std::time::Instant;
 
 #[cfg(test)]
 pub static TEST_MAX_THREADS: AtomicUsize = AtomicUsize::new(100);
 
+fn sleep_ms(ms: u64) {
+    std::thread::sleep(Duration::from_millis(ms));
+}
+
 struct AtomicCounter {
     next_value: AtomicUsize,
 }
@@ -514,7 +520,7 @@ impl ThreadPool {
                     // At least one thread is running.
                 }
                 Err(StartThreadsError::NoThreads(_)) => {
-                    std::thread::sleep(Duration::from_millis(10));
+                    sleep_ms(10);
                     continue;
                 }
             }
@@ -523,7 +529,7 @@ impl ThreadPool {
                 Err(TrySendError::Disconnected(_)) => unreachable!(),
                 Err(TrySendError::Full(box_f)) => Some(box_f),
             };
-            std::thread::sleep(Duration::from_millis(10));
+            sleep_ms(10);
         }
     }
 
@@ -546,6 +552,34 @@ impl ThreadPool {
         };
         self.inner.start_threads().map_err(std::convert::Into::into)
     }
+
+    /// Consumes the thread pool and waits for all threads to stop.
+    pub fn join(self) {
+        let inner = self.inner.clone();
+        drop(self);
+        while inner.num_live_threads() > 0 {
+            sleep_ms(10);
+        }
+    }
+
+    /// Consumes the thread pool and waits for all threads to stop.
+    ///
+    /// # Errors
+    /// Returns an error if the threads do not stop within the timeout duration.
+    pub fn try_join(self, timeout: Duration) -> Result<(), String> {
+        let inner = self.inner.clone();
+        drop(self);
+        let deadline = Instant::now() + timeout;
+        loop {
+            if inner.num_live_threads() < 1 {
+                return Ok(());
+            }
+            if deadline < Instant::now() {
+                return Err("timed out waiting for ThreadPool workers to stop".to_string());
+            }
+            sleep_ms(10);
+        }
+    }
 }
 impl Debug for ThreadPool {
     fn fmt(&self, f: &mut Formatter<'_>) -> Result<(), core::fmt::Error> {
@@ -585,10 +619,6 @@ mod test {
         TEST_MAX_THREADS.store(n, Ordering::Release);
     }
 
-    fn sleep_ms(ms: u64) {
-        std::thread::sleep(Duration::from_millis(ms));
-    }
-
     fn panic_threads(pool: &ThreadPool, num: usize) {
         let pause = Arc::new(AtomicBool::new(true));
         for _ in 0..num {
